#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "usart.h"

#define TRIGGER 	PB3
#define ECHO		PB4
#define LED 		PD7

#define PRESCALER	8
#define MAX			0xFFFF

static volatile int timer 	= 0;
static volatile int pulse 	= 0;
volatile long count = 0;

ISR(TIMER1_OVF_vect)
{
	if(pulse) timer++;
}

int main(void)
{
	char text[50];

	USART0_init();
	DDRD |= (1 << LED);
	DDRB |= (1 << TRIGGER);
	DDRB &= ~(1 << ECHO);

	PORTB &= ~(1 << TRIGGER);

	for (int i = 0; i < 3; i++) {
		PORTD |= (1 << LED);
		_delay_ms(500);	
		PORTD &= ~(1 << LED);
		_delay_ms(500);
	}

	TCCR1B |= (1 << CS11);		
	TIMSK1 |= (1 << TOIE1);
	sei();

	while(1)
	{
		//trigger
		PORTB |= (1 << TRIGGER);
		_delay_us(12);
		PORTB &= ~(1 << TRIGGER);

		// echo
		timer = 0;
		while ((PINB & (1 << ECHO)) == 0);
		TCNT1 = 0;
		
		pulse = 1;
		while ((PINB & (1 << ECHO)) != 0);
		pulse = 0;
		count = (TCNT1 + (MAX * timer));

		if (count * PRESCALER < 2000)
			sprintf(text, " Too close.\n");
		else if (count * PRESCALER > 400000)
			sprintf(text, " Too far.\n");
		else
			sprintf(text, " Distance: %ld.%ld cm\n",
				count * PRESCALER / 1000,
				(count * PRESCALER) % 1000 / 10);
		USART0_print(text);

		PORTD |= (1 << LED);
		_delay_ms(100);
		PORTD &= ~(1 << LED);
	}
	return 0;
}